-- ARPlayDemo
app = AR:create_application(0, "ARPlayDemo")
app:load_scene_from_json("res/main_scene.json","ARPlayDemo")
scene = app:get_current_scene()
root = scene:get_root_node()
screen_size = app:get_property_vec2("screen_size")

local initFinish = false 

local horizontal_plane_dic = {}
local vertical_plane_dic = {}
local plane_position_dic = {}

-- loading_finish --
app.on_loading_finish = function()
    ARLOG("on_loading_finish")
    init_fsm()
    init()
end

function init()

    fsm:set_status("initial")
    fsm:set_status("start_detect")

    -- 增加各种事件的埋点
    Event:addEventListener("plane_feature_info", plane_info)
    Event:addEventListener("plane_anchor_added", plane_anchor_added)
    Event:addEventListener("plane_anchor_updated", plane_anchor_updated)
    Event:addEventListener("plane_anchor_removed", plane_anchor_removed)

end

-- -=-=-=-=-==-=-=-=-=-=状态机初始化及状态函数-=-=-=-=-=-==-=-=-=-=-=-=-=-==-
function init_fsm()
    FsmStatus = {
        {"uninitial", "initial", on_initial_status},
        {"initial", "start_detect", on_start_detect_status},
        {"start_detect", "detected_plane", on_detected_status}
    }
    fsm = FSM()
    fsm:init(FsmStatus)
    fsm:set_status("uninitial")
end

-- 初始化状态
function on_initial_status()
    show_initial_view()
end

-- 检测状态
function on_start_detect_status()
    play_start_music()
    initFinish = true
    simulateLoadProgressUpdate(2000, true)
    AR:perform_after(2000, start_detect_plane)
end

-- 检测平面完成状态
function on_detected_status()
    show_begin_view()
end

-- 游戏中状态
function on_playing_status()

    -- 显示游戏进行中ui
    show_play_view()

end

function on_restart_status()
    show_detect_view()

    -- researt arkit
    Slam.redetect_plane()
end

function on_end_status()
    -- 显示最终结束画面
    ARLOG("set_end score = "..end_score)
end

-- -=-=-=-=-==-=-=-=-=-=按钮的点击事件-=-=-=-=-=-==-=-=-=-=-=-=-=-==-
scene.start_game.on_click = function()
    fsm:set_status("playing_game")
end

scene.start_detect.on_click = function()
    -- 设置平面检测类型
    Slam.set_plane_detection_type(DetectionType.VerticalAndHorizontal)
    -- 开始检测平面
    Slam.start_plane_detect()
    show_detect_view()
end

scene.exit.on_click = function ()
    -- fsm:set_status("detected_plane")
    SingleAbility.close_ar()
end

-- -=-=-=-=-=-=-=-=-=-=- 每一帧的回调 -=-=-=-=--=--=-=-=--=-=-
scene.root.on_update = function()

end

-- 开始检测平面 --
function start_detect_plane()
     -- 设置平面检测类型
     ARKit.set_plane_detection_type(DetectionType.VerticalAndHorizontal)
     -- 开始检测平面 
     ARKit.start_plane_detect()
     show_detect_view()
end

-- -=-=-=-=-=-=-=-=-=-=-平面检测信息的回调-=-=-=-=--=--=-=-=--=-=
--- 平面特征点信息回调 -- 
function plane_info(event)

end

-- anchor 找到的事件
first_detect = false
function plane_anchor_added(event)
    ARLOG("plane_anchor_added")
    show_detected_view()

    local identifier = event.data['identifier']
    local plane_type = event.data['plane_type']
    local extent = event.data['extent']
    local center = event.data['center']
    local transform = event.data['transform']
    local position = event.data['position']

    scene.monster_b:set_property_bool("visible", true)

end

--- anchor 更新的事件 ---
function plane_anchor_updated(event)  
    ARLOG("plane_anchor_updated")
end

---  anchor 被移除的事件 ---
function plane_anchor_removed(event)
    ARLOG("plane_anchor_removed")
end


