controller = {
    _bulletDic = {}, -- 子弹数组
    _shoot_id = 0,
    _time = 0,
    _touch = {}
}
-- button类型
ButtonType = {
    shoot = 1,  -- 发射子弹,对应json对象名shoot_left
    boom = 2    -- 发射炸弹,对应json对象名shoot_right
}
-- 子弹状态
BulletStatus = {
    living = 1,      -- 飞行状态
    collision = 2,   -- 碰撞爆炸状态
    dead = 3         -- 消亡状态
}

-- 子弹生产
function controller:bullet_produce(  )
    -- produce
    self._shoot_id = self._shoot_id + 1
    local cloned_bullet = scene:clone_node("bullet", tostring(self._shoot_id))
    root:add_sub_node(cloned_bullet)
    local bullet_object = { _entity = cloned_bullet , _livingStatus = BulletStatus.living, _life = 0 }
    table.insert(self._bulletDic, bullet_object)
    self:bullet_property_set(cloned_bullet)
end

-- 子弹生产后的属性设置(visible,position,velocity)
function controller:bullet_property_set( cloned_bullet )
    if (cloned_bullet == nil) then
        return
    end
    -- visible,rotation    
    local camera_pos = scene:get_active_camera():get_property_vec3("world_position")
    cloned_bullet:set_property_vec3("world_position",camera_pos)
    local rot_offset = ae.ARQuat:new_local(ae.ARVec3:new_local(0,1,0), math.pi / 2)
	local camera_rot = scene:get_active_camera():get_property_quat("world_rotation_quat")
    cloned_bullet:set_property_quat("world_rotation_quat",camera_rot * rot_offset)
	-- impluse
	local screen_pos = app:get_screen_size() * ae.ARVec2:new_local(0.5, 0.5)
    local dest = scene:get_active_camera():unproject(screen_pos, 1000)
    local impluse = (dest - camera_pos):normalize() * 5
    cloned_bullet:create_physics_body(0.5, 0.5, 10,"dynamic", "capsule_z")
    local physics_body = cloned_bullet:get_physics_body()
    physics_body:set_property_vec3("impluse", impluse)
    cloned_bullet:set_property_bool("visible", true)
    create_bullet_fire(ae.ARVec3:new_local(0,0,5))
end
-- 注册button回调
function controller:register_callback(  )
    if self._type == ButtonType.shoot then
        self._btn:set_interaction_event_handler("button_state_changed",button_left_click)
    elseif self._type == ButtonType.boom then
        self._btn:set_interaction_event_handler("button_state_changed",button_right_click)
    end
end
-- 碰撞回调
function collision_callback( name_a, name_b, pos_a, pos_b, point_num )
    ARLOG(name_a .." "..name_b)
    if(name_b == "monster_group") then
        AR:perform_after(1000, function()
            if (cloned_bullet == nil) then
                return
            end
            local pos = cloned_bullet:get_property_vec3("world_position")
            local extent = cloned_bullet:get_bounding_box():half_extent()
            create_multi_explosion(pos, extent, 5, 0)         
            local  name = cloned_bullet:get_property_string("name")
            scene:remove_node_by_name(name)
            cloned_bullet = nil
        end)
    end
end
-- clone
function controller:clone( object )
    local lookup_table = {}
    local function copyObj( object )
        if type( object ) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        
        local new_table = {}
        lookup_table[object] = new_table
        for key, value in pairs( object ) do
            new_table[copyObj( key )] = copyObj( value )
        end
        return setmetatable( new_table, getmetatable( object ) )
    end
    return copyObj( object )
end
-- 子弹生命周期管理
function controller:bullet_clear(  )
    for i=#self._bulletDic,1,-1 do
        if self._bulletDic[i]._livingStatus == BulletStatus.dead then
            local  name = self._bulletDic[i]._entity:get_property_string("name")
            scene:remove_node_by_name(name)
            self._bulletDic[i]._entity = nil
            table.remove(self._bulletDic,i)
        end
    end
end


function controller:bullet_update(delta)
    for i,v in ipairs(self._bulletDic) do
        v._life = v._life + delta
        local pos_bullet = v._entity:get_property_vec3("world_position")
        create_bullet_fire(pos_bullet)
        if (v._life > 2000) then
            v._livingStatus = BulletStatus.dead
        end
    end
end

function controller:bullet_all_clear()
    for i=#self._bulletDic,1,-1 do
        local  name = self._bulletDic[i]._entity:get_property_string("name")
        scene:remove_node_by_name(name)
        self._bulletDic[i]._entity = nil
        table.remove(self._bulletDic,i)
    end
    
end

-- allbullet_collsion
function controller:check_collision( dragon, delta )
    local dragon_extent = dragon:get_bounding_box():half_extent()
    local pos_dragon = dragon:get_bounding_box():center()
    for i,v in ipairs(self._bulletDic) do
        local pos_bullet = v._entity:get_property_vec3("world_position")
        local is_collsion = self:collision(pos_bullet, pos_dragon, dragon_extent)
        if is_collsion then
            local pos = v._entity:get_property_vec3("world_position")
            v._livingStatus = BulletStatus.dead 
            return is_collsion, pos
        end
    end
end
-- 碰撞检测
function controller:collision( pos_bullet, pos_dragon,dragon_extent )
    local vec = controller:distance(pos_bullet,pos_dragon)
    -- if dis < 0.13 then
    --     return true
    -- end
    -- return false
    if math.abs( vec.z ) > dragon_extent.z then
        return false
    end
    if math.abs( vec.x ) > dragon_extent.x then
        return false
    end
    if math.abs( vec.y ) > dragon_extent.y then
    return false
end
    return true
end
-- distance
function controller:distance( p1, p2 )
    local p = p1 - p2
    --local dis = math.sqrt((p1.x-p2.x)*(p1.x-p2.x) + (p1.y-p2.y)*(p1.y-p2.y) + (p1.z-p2.z)*(p1.z-p2.z))
    return p
end
