-- ~*~*~*~*~*~*火箭炸弹发射器*~*~*~*~*~*~*~*~
-- 发射一颗火箭,

MISSILE_INDEX = 0
MISSILE_NAME_IN_JSON = "missile"
BONUS_NAME_IN_JSON = "bonus"

missile_list = {}

function create_missile_instance(eg_node, name, lifetime, is_bonus)
    local missile_instance = {
        eg_node = nil,
        name = "",
        lifetime = 0,
        damage = 250,
        should_explosion = false,
        explosion_extent = 1.0,
        is_alive = true,
        music_launch_session = nil,
        is_bonus_missile = false,

        update = function(self, delta)
            if (self.is_bonus_missile) then
                self:update_bonus_missile(delta)
            else 
                self:update_no_bonus_missile(delta)
            end
            
            
        end,

        update_bonus_missile = function(self, delta)
            local pos = self:center()

            local camera_pos = scene:get_active_camera():get_property_vec3("world_position")

            local distance = pos - camera_pos

            if (math.abs(distance.x) < 0.5 and math.abs(distance.z) < 0.5) then
                player:add_missile(1)
                scene:remove_node_by_name(self.name)
                self.eg_node = nil
                self.is_alive = false
            end
        end,

        update_no_bonus_missile = function(self, delta)

            self.lifetime = self.lifetime - delta

            if (self.lifetime < 0) then
                self:explosion()
            else 
                --正常
                self:check_explosion()
            end

        end,

        check_explosion = function(self)
            local missile_center = self.eg_node:get_bounding_box():center()

            for k,v in pairs(monster_list) do
                if (v.is_alive) then
                    local monster_center = v.eg_node:get_bounding_box():center()
                    local monster_extent = v.eg_node:get_bounding_box():half_extent()
                    local distance = monster_center - missile_center
                    if (math.abs(distance.x) < monster_extent.x and
                        math.abs(distance.y) < monster_extent.y and
                        math.abs(distance.z) < monster_extent.z) then

                        self:explosion()

                        break
                    end
                end
            end
        end,

        explosion = function(self)
            self:do_damage_to_monster()
            local pos = self.eg_node:get_bounding_box():center()
            local extent = eg_node:get_bounding_box():half_extent()
            create_multi_explosion(pos, extent, 5, 300)
            --播放爆炸音乐
            play_missile_explosion_music(self.eg_node, 5 * 300)
            --停止发射音乐
            self.music_launch_session:stop()

            self.eg_node:set_property_bool("visible", false)
            self.is_alive = false
        end,

        world_position = function(self)
            local  pos = self.eg_node:get_property_vec3("world_position")
            return pos
        end,

        center = function(self)
            local bounding_box = self.eg_node:get_bounding_box()
            local center = bounding_box:center()
            return center

        end,

        do_damage_to_monster = function(self)
            local missile_pos = self.eg_node:get_bounding_box():center()
            
            for k,v in pairs(monster_list) do
                if (v.is_alive) then
                    local monster_pos = v.eg_node:get_bounding_box():center()
                    local distance = monster_pos - missile_pos
                    local abs_dis = distance:length()
                    ARLOG("do_damage_to_monster distance "..abs_dis)
                    if (abs_dis < self.explosion_extent) then
                        v:receive_damage(self.damage)
                    end

                end
            end

        end
    }

    missile_instance.eg_node = eg_node
    missile_instance.name = name
    missile_instance.lifetime = lifetime
    missile_instance.is_bonus_missile = is_bonus
    if (not is_bonus) then
        missile_instance.music_launch_session = play_launch_missile_music(eg_node)
    end
    return missile_instance

end


function launch_missile(lifetime)
    local node, name = clone_missile_in_engine(lifetime)
    local missile = create_missile_instance(node, name, lifetime, false)
    table.insert(missile_list, missile)
end

function missile_list_update(delta)
    for k,v in pairs(missile_list) do
        if (v.is_alive) then
            v:update(delta)
        else
            table.remove(missile_list, k)
        end
    end
end


function clone_missile_in_engine(time)
	local missile_node = scene:clone_node(MISSILE_NAME_IN_JSON,tostring(MISSILE_INDEX))
	MISSILE_INDEX = MISSILE_INDEX + 1
	root:add_sub_node(missile_node)
	missile_node:set_property_bool("visible", true)

    --火箭位置接近相机，并有一个pos_offset
	local camera_pos = scene:get_active_camera():get_property_vec3("world_position")
	ARLOG("launch_missile camera_pos:"..camera_pos:to_string())
	local pos_offset = ae.ARVec3:new_local(-0.2,0.2,0.05)
	missile_node:set_property_vec3("world_position",camera_pos + pos_offset)

    --火箭朝向与相机一致，由于模型坐标系原因需要乘以rot_offset
    local rot_offset = ae.ARQuat:new_local(ae.ARVec3:new_local(0,1,0), math.pi)
	local camera_rot = scene:get_active_camera():get_property_quat("world_rotation_quat")
    missile_node:set_property_quat("world_rotation_quat",camera_rot * rot_offset)
	
	local screen_pos = app:get_screen_size() * ae.ARVec2:new_local(0.5, 0.5)
    local dest = scene:get_active_camera():unproject(screen_pos, 1000)
    local impluse = (dest - camera_pos):normalize() * 6.0

    missile_node:create_physics_body(1.0, 0.5, 10,"dynamic", "capsule_z")
    local physics_body = missile_node:get_physics_body()
    physics_body:set_property_vec3("impluse", impluse)

    --physics_body:set_collision_handler(missile_collision_handler)
    
    local name = missile_node:get_property_string("name")
    return missile_node, name
end


-- function missile_collision_handler(name_a, name_b, pos_a, pos_b, num)

--     ARLOG("missile_collision_handler:"..name_a.." "..name_b)

--     for k,v in pairs(missile_list) do
--         if (v.name == name_a) then
--             v.should_explosion = true
--         end
--     end
-- end


function play_launch_missile_music(node)
    if (node == nil) then
        return
    end
    local config = {}
    config["repeat_count"] = -1
    config["is_remote"] = 0
    local media_session = node:get_media_controller():create_media_session("audio", "res/media/weapon/missile_fly.mp3", config)

    media_session:play()

    return media_session
end

function play_missile_explosion_music(node, time)
    if (node == nil) then
        return
    end
    local config = {}
    config["repeat_count"] = -1
    config["is_remote"] = 0
    local media_session = node:get_media_controller():create_media_session("audio", "res/media/weapon/missile_explosion.mp3", config)

    media_session:play()

    local name = node:get_property_string("name")

    AR:perform_after(time, function()
        if (node == nil) then
            return
        end
        media_session:stop()
        scene:remove_node_by_name(name)
        node = nil
    end)

end

function missile_list_reset(delta)
    
    for i = #missile_list, 1, -1 do
        if missile_list[i] ~= nil then

            local name = missile_list[i].name
            scene:remove_node_by_name(name)
            missile_list[i].eg_node = nil
            table.remove(missile_list, i)
        end
    end
end
