-- 产生怪物的函数
--[[

eg_node :引擎的节点 ARNode

type : 
"A":钢甲虫，特点血量高无攻击，初始化后就开始拼命撞向玩家。血量：500  速度：20 攻击：10
"B":钢甲攻击虫，特点血量高攻击高，初始化后开始飞向玩家，在固定范围内进行子弹攻击。血量：300 速度：10 攻击：15
"C":无甲攻击虫，特点血量低攻击高，初始化后开始飞向玩家，在固定范围内进行子弹攻击。血量：100 速度：10 攻击：25

name:引擎中的名字

]]
local MONSTER_INDEX = 0
local A_MONSTER_NAME_IN_JSON = "monster_a"
local B_MONSTER_NAME_IN_JSON = "monster_b"
local C_MONSTER_NAME_IN_JSON = "monster_c"
monster_list = {}

function create_monster_instance(node, type, name, speed, attack, total_life, now_life, score, first_move_position)
	local monster_instance = {
	    eg_node = nil,
        fly_anim_session = nil,
        dash_session = nil,
        first_move_session = nil,
        random_move_session = nil,
        type = "",
        name = "",
        speed = 0,
        attack = 0,
        total_life = 0,
        now_life = 0,
        score = 0,
        is_alive = true,
        fsm_control = nil,

        fire_gap = 8000,
        fire_gap_count = 0,
        dash_gap = 30000,
        dash_gap_count = 0,
        random_move_gap = 10000,
        ramdom_move_gap_count = 0,

        init_fsm = function(self)
            self.fsm_control = FSM()

            local ta = {

            {"on_create", "on_move", function() self:on_move_trigger() end},
            {"on_move", "on_fire", function() self:on_fire_trigger() end},
            {"on_move", "on_ramdom_move", function() self:on_ramdom_move_trigger() end},
            {"on_move", "on_dash", function() self:on_dash_trigger() end}

            }

            self.fsm_control:init(ta)
            self.fsm_control:set_status("on_create")
        end,

        get_fsm_state = function(self)
            return self.fsm_control:current_status()
        end,

        on_move_trigger = function(self)
            ARLOG("status from on_create to on_move")
        end,

        on_fire_trigger = function(self)
            ARLOG("status from on_move to on_fire")

            self:fire(self.fire_gap * 0.8)
        end,

        on_ramdom_move_trigger = function(self)
            ARLOG("status from on_move to on_ramdom_move")

            self:ramdom_move(self.random_move_gap * 0.8)
            self:fire(self.fire_gap * 0.8)
        end,

        on_dash_trigger = function(self)
            ARLOG("status from on_move to on_dash")

            self:dash_to_player()
        end,

        change_state_after_on_move = function(self)
        	ARLOG("change_state_after_on_move call")

            if (not self.is_alive) then
                ARLOG(" error:when call change_state_after_on_move, node is not alive!!!!")
                return
            end

            if (self.type == "A") then

                self.fsm_control:set_status("on_dash")

            elseif (self.type == "B") then

                self.fsm_control:set_status("on_fire")

            elseif (self.type == "C") then

                self.fsm_control:set_status("on_ramdom_move")
            end

        end,

        update = function(self, delta)

            if (self:get_fsm_state() ~= "on_create" and 
                self:get_fsm_state() ~= "on_move") then

                self:rotate_to_player()

            end

            if (self:get_fsm_state() == "on_ramdom_move") then
                self:ramdom_move(delta)
            end

            if (self:get_fsm_state() ~= "on_create" and 
                self:get_fsm_state() ~= "on_move" and 
                self:get_fsm_state() ~= "on_dash") then

                self:fire(delta)
            end

            if (self:get_fsm_state() == "on_dash") then
                self:dash(delta)
            end
            
	        self:dead()
	    end,

        rotate_to_player = function(self)
            local player_pos = player:world_position()
            local pos = self:world_position()
            local dir = player_pos - pos
            dir = dir:normalize()
            local quat = billboard(dir)
            self.eg_node:set_property_quat("world_rotation_quat", quat)
        end,

        dash = function(self, delta)
            local pos = self:world_position()
            if (player:is_hitted(pos)) then
                player:receive_damage(self.attack)
                self.now_life = -1
                return
            end

            self.dash_gap_count = self.dash_gap_count + delta

            if (self.dash_gap_count > self.dash_gap) then
                self.dash_gap_count = 0.0
                self:dash_to_player()
            end

        end,

        dash_to_player = function(self)

            local pos = self:world_position()

            local player_pos = player:world_position()

            local anim_config = {}

            anim_config["rigid_anim_type"] = "translate"
            anim_config["translate_from"] = pos
            anim_config["translate_to"] = player_pos
            anim_config["repeat_count"] = 1

            self.dash_session = self.eg_node:get_animation_controller():create_animation_session("rigid", anim_config)
            self.dash_session:play()

        end,

        fire = function(self, delta)

            self.fire_gap_count = self.fire_gap_count + delta

            if (self.fire_gap_count > self.fire_gap) then
                local monster_pos = self:world_position()
                local camera_pos = scene:get_active_camera():get_property_vec3("world_position")

                local ammo_count = math.random(-1, 1)

                if (ammo_count < 0) then
                    ammo_count = 1
                else
                    ammo_count = 3
                end

                local attack = self.attack
                create_monster_weapon("monster_weapon_a", monster_pos, camera_pos, ammo_count, attack)

                self:play_fire_music()
                self.fire_gap_count = 0.0
            end
        end,

	    ramdom_move = function(self, delta)
	        --math.randomseed(os.time())

            self.ramdom_move_gap_count = self.ramdom_move_gap_count + delta

            if (self.ramdom_move_gap_count > self.random_move_gap) then
	            local x = math.random(-100,100) / 100
                local y = math.random(-100,100) / 100
                local z = math.random(-100,100) / 100

                local pos = self:world_position()

                local ramdom_pos = pos + ae.ARVec3:new_local(x, y, z)

                self.random_move_session = MonsterMove.new(self.eg_node, scene, 1.0, 20)
                self.random_move_session:execute("move", ramdom_pos)

                self.ramdom_move_gap_count = 0.0

                ARLOG("ramdom_move pos "..ramdom_pos:to_string())
            end


	    end,

        receive_damage = function(self, damage)
            if (self.is_alive) then
                self.now_life = self.now_life - damage
                self:play_bleeding_music()
                ARLOG("life :"..self.now_life)
            end
        end,

        dead = function(self)
            if (self.now_life <= 0) then
                -- 删除雷达节点
                remove_node_in_radar(self)
                
            	--播放死亡效果再删除
                self:play_dying_music()
                self.fly_anim_session:stop()

                if (self.dash_session ~= nil) then
                    self.dash_session:stop()
                end

                if (self.random_move_session ~= nil) then
                    self.random_move_session:stop()
                end

                if (self.first_move_session ~= nil) then
                    self.first_move_session:stop()
                end

                self:rotate_and_drop_down()
            	self.is_alive = false

                -- 增加分数
                player:receive_scroe(self.score)
            end
        end,

        world_position = function(self)
            local  pos = self.eg_node:get_property_vec3("world_position")
            return pos
        end,

        play_bleeding_music = function(self)
            if self.eg_node == nil then
                return
            end
            local config = {}
            config["repeat_count"] = 1
            config["is_remote"] = 0
            config["forward_conflict_solving_strategy"] = "cancel_all_forward"
            local media_session = self.eg_node:get_media_controller():create_media_session("audio", "res/media/monster/monster_bleed_1.mp3", config)
            media_session:play() 

        end,

        play_dying_music = function(self)
            if self.eg_node == nil then
                return
            end
            local config = {}
            config["repeat_count"] = 1
            config["is_remote"] = 0
            local media_session = self.eg_node:get_media_controller():create_media_session("audio", "res/media/monster/monster_die_1.mp3", config)

            media_session:set_state_handler(function()
                if (self.type == "C") then
                    self:create_bonus_missile()
                end
                
                -- ARLOG("remove_node_by_name "..self.name)
                scene:remove_node_by_name(self.name)
                self.eg_node = nil
            end)
            media_session:play() 
        end,

        play_fire_music = function(self)
            if self.eg_node == nil then
                return
            end
            local config = {}
            config["repeat_count"] = 1
            config["is_remote"] = 0
            local media_session = self.eg_node:get_media_controller():create_media_session("audio", "res/media/monster/monster_fire.mp3", config)
            media_session:play() 
        end,
        
        rotate_and_drop_down = function(self)

            local gravity_y = 2
            local gravity = ae.ARVec3:new_local(0, -1 * gravity_y, 0)

            self.eg_node:create_physics_body(100.0, 0.0, 10,"dynamic", "sphere")
            self.eg_node:get_physics_body():set_property_vec3("gravity", gravity)

            --self.eg_node:set_property_bool("physics/sync_physics_rotation",true)

            local range = math.pi

            local torque_impluse = ae.ARVec3:new_local(math.pi, math.pi, math.pi)
            self.eg_node:get_physics_body():set_property_vec3("angular_velocity", torque_impluse)

            -- local config = {}
            -- config["repeat_count"] = 1
            -- config["start_frame"] = 106
            -- config["end_frame"] = 135
            -- local model_anim_session = self.eg_node:create_model_animation(config)
            -- model_anim_session:set_state_handler(function()
            --      scene:remove_node_by_name(self.name)
            --      self.eg_node = nil

            -- end)
            -- model_anim_session:play()





            -- local world_pos = self:world_position()
            -- local y = math.abs(world_pos.y)

            -- local t = math.sqrt(2 * y / gravity_y)

            -- AR:perform_after(t * 1000, function()
            --     scene:remove_node_by_name(self.name)
            -- end)
        end,

        create_bonus_missile = function(self)

            local position = self:world_position()
            local missile_node = scene:clone_node(BONUS_NAME_IN_JSON, tostring(MISSILE_INDEX))
            MISSILE_INDEX = MISSILE_INDEX + 1
            root:add_sub_node(missile_node)
            missile_node:set_property_bool("visible", true)

            local camera_pos = scene:get_active_camera():get_property_vec3("world_position")
            position.y = camera_pos.y
            missile_node:set_property_vec3("world_position", position)

            local name = missile_node:get_property_string("name")

            local missile = create_missile_instance(missile_node, name, 10000, true)
            table.insert(missile_list, missile)

            ARLOG("create bonus missile "..name)
        end,

        --怪物出洞以后先到洞上方某个位置
        create_first_move_session = function(self, move_position)
            -- 出洞到目标位置,更新怪兽状态为可以可以参与update的更新逻辑
            self.first_move_session = MonsterMove.new(self.eg_node, scene, 0.3, 20)
            self.first_move_session:execute("move_rotate_camera", move_position)

            self.first_move_session:set_complete_handler("rotate", 
                function() 

                    AR:perform_after(100, function()
                        self:change_state_after_on_move() 
                    end)

                end)
        end,
    }
  	monster_instance.eg_node = node
  	monster_instance.type = type
  	monster_instance.name = name
	monster_instance.speed = speed
	monster_instance.attack = attack
	monster_instance.total_life = total_life
	monster_instance.now_life = now_life
    monster_instance.score = score
    monster_instance.fly_anim_session = play_monster_fly_anim(node)

    monster_instance:create_first_move_session(first_move_position)
    monster_instance:init_fsm()

  	return monster_instance
end

local PORTAL_DISTANCE = 0.5
local MAX_PORTAL_COUNT = 4
potential_portal_position = {}

local HOLE_INDEX = 0
local HOLE_NAME_IN_JSON = "portal_a"
portal_list = {}

function create_portal_instance(node, name, ability, move_positon)

	local portal_instance = {
	    eg_node = nil,
	    name = name,
	    ability = 0,
	    birth_delta_time = 0,
	    birth_time = 4000, -- 4秒出现一个怪物
	    is_alive = true,
        move_position = {},
        monster_type = {"A", "B", "C", "C"},

	    update = function(self, delta)
	        self:create_one_monster(delta)
	        self:dead()
	    end,

        create_one_monster = function(self, delta)
            self.birth_delta_time = self.birth_delta_time + delta
            if (self.birth_delta_time > self.birth_time) then

                local pos = self:world_position()
                local trans = self.eg_node:get_property_mat44("world_transform")
                local up_dir = trans:up()

                --从表中取一个元素，并移除
                local num = #self.monster_type
                local index = math.random(1, num)
                local monster_type = self.monster_type[index]

            	create_monster(monster_type, pos, up_dir, move_positon[1])

                ARLOG("create_monster "..self.name)

                table.remove(move_positon, 1)
                table.remove(self.monster_type, index)

            	self.ability = self.ability - 1
            	self.birth_delta_time = 0
            end
        end,

        dead = function(self)
            if (self.ability <= 0) then

            	--TODO:传送门消失的特效
            	scene:remove_node_by_name(self.name)
                self.eg_node = nil
            	self.is_alive = false
                check_game_resource(true, nil, nil)
            end
        end,

        world_position = function(self)
            local  pos = self.eg_node:get_property_vec3("world_position")
            return pos
        end
    }
	portal_instance.eg_node = node
	portal_instance.name = name
	portal_instance.ability = ability
	return portal_instance
end


-------可调用的方法------
function create_portal_with_table(plane_table)
    for i,v in pairs(plane_table) do
        local plane_type = v["plane_type"]
        local place_pos = v["position"]
        local transform = v["transform"]
        if (place_pos ~= nil) then
            local normal = ae.ARVec3:new_local(0,1,0)
            if (plane_type == "vertical") then
            	local temp_transform = transform
                temp_transform.w = ae.ARVec4:new_local(0,0,0,1)
                local temp_vec4 = temp_transform * ae.ARVec4(normal.x,normal.y,normal.z,1)
                normal = ae.ARVec3:new_local(temp_vec4.x,temp_vec4.y,temp_vec4.z)
                normal = normal:normalize()
            end
            create_portal(place_pos, normal)
        end
    end
end

--检查当前全局洞穴数量数量
--如果当前洞穴数量已经>=最大数量，则新的洞穴位置保存待以后使用
--如果当前洞穴数量<最大数量，则直接生成洞穴开始出怪物
--调用时机：平面扫描到新平面/扫描到符合大小的新位置和某个洞穴销毁时

function check_game_resource(called_by_destory, world_position, up_dir)
    world_position = world_position or ae.ARVec3:new_local()
    up_dir = up_dir or ae.ARVec3:new_local()
    local cur_portal_count = get_table_count(portal_list)
    if (called_by_destory) then
        if (cur_portal_count < MAX_PORTAL_COUNT) then
            if (get_table_count(potential_portal_position) > 0) then
                for k,v in pairs(potential_portal_position) do
                    create_portal(k, v)
                    potential_portal_position[k] = nil
                    break
                end
            end
        end
    else
        if (cur_portal_count < MAX_PORTAL_COUNT) then
            create_portal(world_position, up_dir)
        else
            potential_portal_position[world_position] = up_dir
        end
    end
end

function create_portal(world_position, up_dir)

    for i,v in ipairs(portal_list) do
        local current_position = v:world_position()
        local d = point_distance(current_position, world_position)
        if d < PORTAL_DISTANCE then 
            return 
        end
    end

	local eg_node, name = clone_portal_in_engine(HOLE_NAME_IN_JSON, world_position, up_dir)
    
    -- 根据模型初始位置和up dir确定固定位置队列
    local monster_move_positions = generate_random_monster_position(world_position, up_dir, 4, 0.5, 0.4)

    -- local monster_move_positions = {}
    -- local height_to_portal = 0.2
    -- local distance_to_hcenter = 0.2
    -- local height_center = world_position + up_dir * height_to_portal

    -- -- temp:4个位置，供创建怪兽时选择
    -- table.insert(monster_move_positions, height_center + ae.ARVec3:new_local(-1*distance_to_hcenter, 0, -1*distance_to_hcenter))
    -- table.insert(monster_move_positions, height_center + ae.ARVec3:new_local(-1*distance_to_hcenter, 0, distance_to_hcenter))
    -- table.insert(monster_move_positions, height_center + ae.ARVec3:new_local(distance_to_hcenter, 0, -1*distance_to_hcenter))
    -- table.insert(monster_move_positions, height_center + ae.ARVec3:new_local(distance_to_hcenter, 0, distance_to_hcenter))

    -- ARLOG("zy create_portal world_position = "..world_position:to_string().." up_dir = "..up_dir:to_string())
    -- for k,v in pairs(monster_move_positions) do
    --     ARLOG("zy monster_move_positions with key = "..k.." v = "..v:to_string())
    -- end

	local portal = create_portal_instance(eg_node, name, 4, monster_move_positions)
	table.insert(portal_list, portal)
end

function create_monster(monster_type, pos, up_dir, move_position)

	local monster = nil


	if (monster_type == "A") then
        ARLOG("create monster a")
		local eg_node, name = clone_monster_in_engine(A_MONSTER_NAME_IN_JSON, pos, up_dir)
		local monster = create_monster_instance(eg_node, "A", name, 20, 10, 500, 500,10, move_position)

        monster.fsm_control:set_status("on_move")

        add_node_in_radar(monster)
		table.insert(monster_list, monster)

	elseif (monster_type == "B") then
        ARLOG("create monster b")
		local eg_node, name = clone_monster_in_engine(B_MONSTER_NAME_IN_JSON, pos, up_dir)
		local monster = create_monster_instance(eg_node, "B", name, 10, 15, 300, 300,20, move_position)

        monster.fsm_control:set_status("on_move")

        add_node_in_radar(monster)
		table.insert(monster_list, monster)

	elseif (monster_type == "C") then
        ARLOG("create monster c")
		local eg_node, name = clone_monster_in_engine(C_MONSTER_NAME_IN_JSON, pos, up_dir)
		local monster = create_monster_instance(eg_node, "C", name, 10, 25, 100, 100,50, move_position)

        monster.fsm_control:set_status("on_move")

        add_node_in_radar(monster)
		table.insert(monster_list, monster)
	end
end

function clone_portal_in_engine(name, pos, up_dir)
	local eg_node = scene:clone_node(name, tostring(HOLE_INDEX))
	HOLE_INDEX = HOLE_INDEX + 1
	root:add_sub_node(eg_node)
	eg_node:set_property_bool("visible", true)

    local quat = ae.ARQuat:new_local()
    local up_abs = ae.ARVec3:new_local(0, 1, 0)
    up_dir:normalize()

    quat:rotation(up_abs, up_dir)

    eg_node:set_property_vec3("position",pos)
    eg_node:set_property_quat("rotation_quat", quat)

	local name = eg_node:get_property_string("name")
	return eg_node, name
end

function clone_monster_in_engine(name, pos, up_dir)
	local eg_node = scene:clone_node(name, tostring(MONSTER_INDEX))
	MONSTER_INDEX = MONSTER_INDEX + 1
	root:add_sub_node(eg_node)
	eg_node:set_property_bool("visible", true)
	eg_node:set_property_vec3("world_position",pos)

	--创建物理碰撞体
	-- eg_node:create_physics_body(1.0, 0.0, 10,"dynamic", "sphere")
 --    local physics_body = eg_node:get_physics_body()

    --local velocity = up_dir:normalize() * 1.5
    --physics_body:set_property_vec3("linear_velocity", velocity)
   
    --eg_node:set_property_bool("physics/sync_physics_rotation",false)

	local name = eg_node:get_property_string("name")
	return eg_node, name
end

function play_monster_fly_anim(node)
    local anim_config = {}
    anim_config["repeat_count"] = -1
    anim_config["speed"] = 1
    
    if (node ~= nil) then
        local anim_controller = node:get_animation_controller()
        if (anim_controller ~= nil) then
            ARLOG("create anim_session in play_monster_fly_anim")
            local anim_session = anim_controller:create_animation_session("model", anim_config)
            anim_session:play()
            return anim_session
        end
    end
    return nil
end


function portal_list_update(delta)

	for k,v in pairs(portal_list) do
		if (v.is_alive) then
			v:update(delta)
		else
			table.remove(portal_list, k)
		end
	end
end

function monster_list_update(delta)
	for k,v in pairs(monster_list) do
		if (v.is_alive) then
			v:update(delta)
		else
			table.remove(monster_list, k)
		end
	end
end

-- 根据洞穴位置和up向量，生成备选的n个怪物飞向位置
-- 洞穴up上方一个球，在球内选取位置
function generate_random_monster_position(pos, up, count, height, radius)
    local ret = {}
    local center = pos + up * height
    math.randomseed(os.time())
    for i = 1, count do
        local r = math.random() * radius
        local theta = math.random() * math.pi
        local phi = math.random() * 2 * math.pi
        local p = ae.ARVec3:new_local(r*math.sin(theta)*math.cos(phi), r*math.sin(theta)*math.sin(phi), r*math.cos(theta))
        table.insert(ret, center + p)
    end
    return ret
end


function portal_list_reset()

    for i = #portal_list, 1, -1 do
        if portal_list[i] ~= nil then

            local name = portal_list[i].name
            scene:remove_node_by_name(name)
            portal_list[i].eg_node = nil
            table.remove(portal_list, i)
        end
    end
end

function monster_list_reset()

    for i = #monster_list, 1, -1 do
        if monster_list[i] ~= nil then

            local name = monster_list[i].name
            scene:remove_node_by_name(name)
            monster_list[i].eg_node = nil
            table.remove(monster_list, i)
        end
    end
    
end

function check_portal_count(time, dic)

    if time % 20 == 0 then 
      local count = get_table_count(portal_list)
   
      if count <= 0 then
        local max = #dic
        local index = math.random(1,max)
        local v = dic[index]
        local plane_type = v["plane_type"]
        local place_pos = v["position"]
        local transform = v["transform"]
        if (place_pos ~= nil) then
            local normal = ae.ARVec3:new_local(0,1,0)
            if (plane_type == "vertical") then
            	local temp_transform = transform
                temp_transform.w = ae.ARVec4:new_local(0,0,0,1)
                local temp_vec4 = temp_transform * ae.ARVec4(normal.x,normal.y,normal.z,1)
                normal = ae.ARVec3:new_local(temp_vec4.x,temp_vec4.y,temp_vec4.z)
                normal = normal:normalize()
            end
            create_portal(place_pos, normal)
        end

      end 
    end
end
