-- 怪兽从洞穴中出来并移动到固定位置，面向玩家，呈现准备好的状态，后续根据怪兽类型
-- 对于C怪兽可能复用这一模块，控制后续每一组的移动和朝向
-- 输入：节点、场景、目标位置、速度、角速度

MonsterMove = class()

function MonsterMove:ctor(node, scene, speed, rotate_speed)
	print("MonsterMove ctor")
	self.node = node
	self.scene = scene
	self.speed = speed or 0.2
	self.rotate_speed = rotate_speed or 0.1
	self.translate_rigid_config = {}
	self.rotate_rigid_config = {}
	self.anim_controller = nil
	self.active_rotate_session = nil
	self.active_translate_session = nil
	self.temp_complete_fun = nil
	self.move_dst = ae.ARVec3:new_local()
	self.rotate_dst = ae.ARVec3:new_local()

	self.anim_controller = self.node:get_animation_controller()
	
	self.translate_rigid_config["rigid_anim_type"] = "translate"
	self.translate_rigid_config["repeat_count"] = 1

	self.rotate_rigid_config["rigid_anim_type"] = "rotate"
	self.rotate_rigid_config["repeat_count"] = 1
end

function MonsterMove:set_move_dst(dst)
	self.move_dst = dst
end

function MonsterMove:set_rotate_dst(dst)
	self.rotate_dst = dst
end

function MonsterMove:rotate_to_camera()
	ARLOG("rotate_to_camera")
	if (self.scene == nil or self.node == nil) then
		return
	end
	local camera_position = self.scene:get_active_camera():get_property_vec3("world_position")
	self.move_dst = camera_position
	self:rotate_to_dst()
end

function MonsterMove:rotate_to_dst()
	ARLOG("rotate_to_dst")
	if (self.node == nil) then
		return
	end
	-- 默认模型在局部坐标系的朝向是z轴正向
	local front_vec = ae.ARVec3:new_local(0, 0, 1)
	local rotation = self.node:get_property_quat("world_rotation_quat")
	local rotate_src = rotation * front_vec

	local position = self.node:get_property_vec3("world_position")
	local rotate_dst = (self.move_dst - position):normalize()

	local quat = ae.ARQuat:new_local()
	quat:rotation(rotate_src, rotate_dst)
	local angle = quat:angle()
	local axis = quat:axis()

	-- local delta_angle = math.deg(rotate_src:vector_angle(rotate_dst))
	-- local rotate_axis = rotate_src:cross(rotate_dst)
	-- if (rotate_axis.y < 0) then
	-- 	delta_angle = -1 * delta_angle
	-- end
	self.rotate_rigid_config["rotate_axis"] = axis
	local delta_angle = math.deg(angle)
	self.rotate_rigid_config["rotate_to"] = delta_angle
	local time = math.floor(math.abs(delta_angle / self.rotate_speed)) * 1000
	self.rotate_rigid_config["duration"] = time

	-- if (self.active_translate_session ~= nil) then
	-- 	self.active_translate_session:stop()
	-- end
	if (self.active_rotate_session) then 
		self.active_rotate_session:stop()
	end
	self.active_rotate_session = self.anim_controller:create_animation_session("rigid", self.rotate_rigid_config)
	self.active_rotate_session:play()
	self.active_rotate_session:set_state_handler(self.temp_complete_fun)
end

function MonsterMove:move_to_dst()
	ARLOG("move_to_dst")
	if (self.node == nil) then
		return
	end
	local move_src = self.node:get_property_vec3("world_position")
	local delta = self.move_dst - move_src
	self.translate_rigid_config["translate_to"] = delta
	local time = math.floor(math.abs(delta:length() * 1000 / self.speed))
	ARLOG("move_to_dst time = "..time.." length = "..delta:length().." speed = "..self.speed)
	self.translate_rigid_config["duration"] = time
	if (self.active_translate_session ~= nil) then
		self.active_translate_session:stop()
	end
	-- if (self.active_rotate_session) then 
	-- 	self.active_rotate_session:stop()
	-- end
	self.active_translate_session = self.anim_controller:create_animation_session("rigid", self.translate_rigid_config)
	self.active_translate_session:play()
end

function MonsterMove:set_complete_handler(rigid_type, handler)
	if (rigid_type == "rotate") then
		if (self.active_rotate_session ~= nil) then
			self.active_rotate_session:set_state_handler(handler)
		else
			self.temp_complete_fun = handler
		end
	elseif (rigid_type == "translate") then
		if (self.active_translate_session ~= nil) then
			self.active_translate_session:set_state_handler(handler)
		end
	end
end

function MonsterMove:execute(move_type, move_dst, rotate_dst)
	move_dst = move_dst or ae.ARVec3:new_local()
	rotate_dst = rotate_dst or ae.ARVec3:new_local()
	self:set_move_dst(move_dst)
	self:set_rotate_dst(rotate_dst)
	if (move_type == "move") then
		self:move_to_dst()
	elseif (move_type == "rotate") then
		self:rotate_to_dst()
	elseif (move_type == "move_rotate") then
		self:move_to_dst()
		self:set_complete_handler("translate", function() 
			AR:perform_after(100, 
				function()
			        self:rotate_to_dst() 
		        end)
			end)
	elseif (move_type == "rotate_move") then
		self:rotate_to_dst()
		self:set_complete_handler("rotate", function() 

			AR:perform_after(100, 
				function()
			        self:move_to_dst() 
		        end)

			end)
	elseif (move_type == "move_rotate_camera") then
		ARLOG("zy MonsterMove:execute with move_dst = "..move_dst:to_string())
		self:move_to_dst()
		self:set_complete_handler("translate", function() 

            AR:perform_after(100, 
				function()
			        self:rotate_to_camera() 
		        end)
			end)
	end
end

function MonsterMove:stop()
	if (self.active_translate_session ~= nil) then
		self.active_translate_session:stop()
	end

	if (self.active_rotate_session ~= nil) then
		self.active_rotate_session:stop()
	end
end
