-- ~*~*~*~*~*~*子弹发射器(使用RayTest)*~*~*~*~*~*~*~*~

local MONSTER_WEAPON_NAME_A = "monster_weapon_a"
local MONSTER_WEAPON_INDEX = 0

monster_weapon_list = {}

function create_monster_weapon_impl(weapon_name, dir, start_pos, attack)

    local eg_node = scene:clone_node(weapon_name, tostring(MONSTER_WEAPON_INDEX))
    MONSTER_WEAPON_INDEX = MONSTER_WEAPON_INDEX + 1
    root:add_sub_node(eg_node)
    eg_node:set_property_bool("visible", true)
    eg_node:set_property_vec3("world_position",start_pos)

    local quat = billboard(dir)
    local quat_offset = ae.ARQuat:new_local(ae.ARVec3:new_local(0,1,0), math.pi / -2.0)

    eg_node:set_property_quat("world_rotation_quat", quat * quat_offset)

    local impluse = dir * 1.2

    eg_node:create_physics_body(1.0, 0.0, 10,"dynamic", "box")
    local physics_body = eg_node:get_physics_body()
    physics_body:set_property_vec3("impluse", impluse)

    eg_node:set_property_bool("physics/sync_physics_rotation",false)

    local monster_weapon_instance = {
        eg_node = nil,
        name = "",
        now_life = 0,
        attack = 0,
        is_alive = true,

        update = function(self, delta)
            self.now_life = self.now_life - delta
            self:do_damage_to_human()
            if (self.now_life <= 0) then
                self:destroy()
            end
        end,

        do_damage_to_human = function(self, delta)
            local pos = eg_node:get_property_vec3("world_position")
            if (player:is_hitted(pos)) then
                player:receive_damage(self.attack)

                self.now_life = -1
            end

        end,

        destroy = function(self)
            scene:remove_node_by_name(self.name)
            self.eg_node = nil
            self.is_alive = false
        end
    }

    monster_weapon_instance.eg_node = eg_node
    monster_weapon_instance.name = eg_node:get_property_string("name")
    monster_weapon_instance.now_life = 10000
    monster_weapon_instance.attack = attack

    table.insert(monster_weapon_list, monster_weapon_instance)
end

function create_monster_weapon(weapon_name, start_pos, target_pos, num, attack)

    for var = 1, num, 1 do

        local front = target_pos - start_pos
        front = front:normalize()
        local up = ae.ARVec3:new_local(0,1,0)
        local left = up:cross(front)
        left = left:normalize()
        up = front:cross(left)
        up = up:normalize()


        local percent = math.pi / (num + 1)

        local quat_offset = ae.ARQuat:new_local(up, -1 * var * percent)

        local dir = quat_offset * left

        --dir = ae.ARVec3:new_local(1,0,1)

        dir = dir:normalize()

        start_pos = start_pos + dir * 0.3

        create_monster_weapon_impl(weapon_name, dir, start_pos, attack)
    end
end


function monster_weapon_list_update(delta)
    for k,v in pairs(monster_weapon_list) do
        if (v.is_alive) then
            v:update(delta)
        else
            table.remove(monster_weapon_list, k)
        end
    end

end

function monster_weapon_list_reset()

    for i = #monster_weapon_list, 1, -1 do
        if monster_weapon_list[i] ~= nil then

            local name = monster_weapon_list[i].name
            scene:remove_node_by_name(name)
            monster_weapon_list[i].eg_node = nil
            table.remove(monster_weapon_list, i)
        end
    end

end
