PlaneController = class()

function PlaneController:ctor()
	print("PlaneController ctor")
	self.planes = {}

	self.SMALL_LIMIT = 0.4
	self.NORMAL_LIMIT = 0.8
	self.BIG_LIMIT = 1.2
	self.SMALL_RADIUS = 0.2
	self.NORMAL_RADIUS = 0.4
	self.BIG_RADIUS = 0.8

	math.randomseed(os.time())
    math.random(0, 360)
end

function PlaneController:reset()
	for k,v in pairs(self.planes) do
		v = nil
	end
end

function PlaneController:create_arkit_plane(plane_type, position, center, extent, transform, node)
	local arkit_plane = {}
	arkit_plane.plane_type = plane_type
	arkit_plane.position = position
	arkit_plane.extent = extent
	arkit_plane.transform = transform
	arkit_plane.center = center
	--UNKNOW,SMALL,NORMAL,BIG
	arkit_plane.scale = "UNKNOW"
	if (node ~= nil) then
		arkit_plane.node = node
	end
	return arkit_plane
end

function PlaneController:update_plane(identifier, plane_type, position, center, extent, transform, node)
	if (self.planes[identifier]) then
		self.planes[identifier].position = position
		self.planes[identifier].extent = extent
		self.planes[identifier].transform = transform
		self.planes[identifier].center = center

		if (self.planes[identifier].node ~= nil) then
			self:update_node(identifier, false)
		end

		local new_scale = self:update_plane_scale(identifier)
		local old_scale = self.planes[identifier].scale

		self.planes[identifier].scale = new_scale
		if ((self:get_scale_number(new_scale) > self:get_scale_number(old_scale)) and (new_scale ~= "UNKNOW") and (new_scale == "BIG")) then
			local origin_pos = self.planes[identifier].center + self:get_random_position(new_scale)
			local ret4 = self.planes[identifier].transform * ae.ARVec4:new_local(origin_pos.x,origin_pos.y,origin_pos.z,1)
			return ae.ARVec3:new_local(ret4.x, ret4.y, ret4.z)
		else 
			return nil
		end
	else
		local arkit_plane = self:create_arkit_plane(plane_type, position, center, extent, transform, node)
		self.planes[identifier] = arkit_plane
		if (self.planes[identifier].node ~= nil) then
			self:update_node(identifier, true)
		end
		local scale = self:update_plane_scale(identifier)
		self.planes[identifier].scale = scale

		local origin_pos = self.planes[identifier].center
		local ret4 = self.planes[identifier].transform * ae.ARVec4:new_local(origin_pos.x,origin_pos.y,origin_pos.z,1)
		return ae.ARVec3:new_local(ret4.x, ret4.y, ret4.z)
		-- if (scale ~= "UNKNOW") then
		-- 	local origin_pos = self.planes[identifier].center + self:get_random_position(scale)
		-- 	local ret4 = self.planes[identifier].transform * ae.ARVec4:new_local(origin_pos.x,origin_pos.y,origin_pos.z,1)
		-- 	return ae.ARVec3:new_local(ret4.x, ret4.y, ret4.z)
		-- else
		-- 	return nil
		-- end
	end
end

function PlaneController:update_plane_scale(identifier)
	local scale = "UNKNOW"
	local plane = self.planes[identifier]
	local extent = plane.extent
	if (extent.x > self.BIG_LIMIT and extent.z > self.BIG_LIMIT) then
		scale = "BIG"
	elseif (extent.x > self.NORMAL_LIMIT and extent.z > self.NORMAL_LIMIT) then
		scale = "NORMAL"
	elseif (extent.x > self.SMALL_LIMIT and extent.z > self.SMALL_LIMIT) then
		scale = "SMALL"
	else 
		scale = "UNKNOW"
	end
	return scale
end

function PlaneController:get_random_position(scale)
	local radius = 0
	if scale == "SMALL" then
		radius = self.SMALL_RADIUS
	elseif scale == "NORMAL" then
		radius = self.NORMAL_RADIUS
	elseif scale == "BIG" then
		radius = self.BIG_RADIUS
	end
	local angle = math.random(0, 360)
	local x = radius * math.sin(math.rad(angle))
	local y = radius * math.cos(math.rad(angle))
	return ae.ARVec3:new_local(x, 0, y)
end

function PlaneController:update_node(identifier, first_add)
	if (self.planes[identifier] == nil) then
		return
	end
	
    local node = self.planes[identifier].node

	local transform = self.planes[identifier].transform
	local extent = self.planes[identifier].extent
	local center = self.planes[identifier].center

	local t = ae.ARVec3:new_local()
    local r = ae.ARQuat:new_local()
    local s = ae.ARVec3:new_local()
    transform:decompose_matrix(t,r,s)
    local axis = r:axis()
    local angle = r:angle()

    extent.y = 1
	local s_pre_mat = ae.ARMat44:create_scale(extent * 0.8)
	local t_pre_mat = ae.ARMat44:create_translation(center)
	local t_post_mat = ae.ARMat44:create_translation(t)
	local r_post_mat = ae.ARMat44:create_rotation(axis, angle)

	local world_trans_mat = t_post_mat * r_post_mat * t_pre_mat * s_pre_mat

	node:set_property_mat44("world_transform", world_trans_mat)
end

function PlaneController:get_scale_number(scale)
	if (scale == "SMALL") then
		return 0
	elseif (scale == "NORMAL") then
		return 1
	elseif (scale == "BIG") then
		return 2
	else 
		return -1
	end
end