Radar = class()

function Radar:ctor(display_node, scene, radius_ratio, position_ratio, valid_radius)
	print("Radar ctor")
	self.node = display_node
	self.scene = scene
	-- (top, left)
	self.radius_ratio = radius_ratio
	self.longer_radius = radius_ratio.y
	
	self.position_ratio = position_ratio
	self.valid_radius = valid_radius or 5
	self.index = 0
	self.targets = {}
end

function Radar:reset()
	for target,hud in pairs(self.targets) do
		if (target ~= nil and hud ~= nil) then
			local dis_name = hud:get_property_string("name")
			self.scene:remove_node_by_name(dis_name)
			self.targets[target] = nil
		end
	end
end

function Radar:add_target(target_node)
	if (target_node == nil) then
		return
	end
	local copy_node = scene:clone_node(self.node:get_property_string("name"), tostring(self.index))
	self.index = self.index + 1
	copy_node:set_property_bool("visible", true)
	scene.root:add_sub_node(copy_node)
	self.targets[target_node] = copy_node
end

function Radar:remove_target(target_node)
	if (target_node == nil) then
		return
	end
	local display_node = self.targets[target_node]
	if (display_node ~= nil and self.scene ~= nil) then
		local dis_name = display_node:get_property_string("name")
		self.scene:remove_node_by_name(dis_name)
		self.targets[target_node] = nil
	end
end

function Radar:update_position()
	local camera = scene:get_active_camera()
	local camera_transform = camera:get_property_mat44("world_transform")
	local origin_front = ae.ARVec4:new_local(0,0,-1,1)
	local target_front = camera_transform * origin_front
	local front = ae.ARVec3:new_local(target_front.x, 0, target_front.z)
	front = front:normalize()

	local camera_position = camera:get_property_vec3("world_position")
	camera_position.y = 0
	-- ARLOG("Radar:update_position")
	for target,hud in pairs(self.targets) do
		if (target ~= nil and hud ~= nil) then

			if (not target.is_alive) then
				return
			end
			-- ARLOG("Radar:update_position with name = "..target:get_property_string("name"))
			local position = target.eg_node:get_property_vec3("world_position")
			position.y = 0
			local distance = (camera_position - position):length()
			-- ARLOG("Radar:distance = "..distance)
			if (distance < self.valid_radius) then
				local ratio = distance / self.valid_radius
				local direction = (position - camera_position):normalize()
				local angle = front:vector_angle(direction)
				local normal = front:cross(direction)
				if (normal.y < 0) then
					angle = angle * -1
				end
				-- ARLOG("Radar:front = "..front:to_string().." direction = "..direction:to_string())
				-- ARLOG("Radar:angle = "..math.deg(angle).." ratio = "..ratio)
				local hud_front = ae.ARVec3:new_local(0,1,0)
				local quat = ae.ARQuat:new_local(ae.ARVec3:new_local(0,0,1), angle)
				local hud_dir = quat * hud_front
				-- ARLOG("Radar:hud_dir = "..hud_dir:to_string())
				local relative_position_vec3 = hud_dir * self.longer_radius * ratio
				local relative_position = ae.ARVec2:new_local(relative_position_vec3.x, relative_position_vec3.y)
				-- ARLOG("Radar:relative_position = "..relative_position:to_string())
				--以left方向获取结果，需要压缩下top方向的数值
				relative_position.x = relative_position.x * self.radius_ratio.x / self.radius_ratio.y
				relative_position = relative_position + self.position_ratio
				
				-- ARLOG("Radar:relative_position = "..relative_position:to_string())
				hud:set_property_bool("visible", true)
				hud:set_hud_position(relative_position.y, relative_position.x)
			else
				hud:set_property_bool("visible", false)
			end
		end
	end
end

