

app = AR:create_application(AppType.Imu, "ARPlayDemo")
app:load_scene_from_json("res/debug_scene.json","ARPlayDemo")
scene = app:get_current_scene()
app.device:open_imu(1)
root = scene:get_root_node()

screen_size = app:get_property_vec2("screen_size")
app:visible_type(ViewVisibleType.HideShotButton)


-- loading_finish --
app.on_loading_finish = function()
    ARLOG("debug on_loading_finish")

    initialize_physics()

    init_touch_controller()

    init_shot_controller()

    init_radar()

    scene:set_update_handler(engine_update)

    score_board = scene.score_board:webview():width(800):height(400):is_remote(0):url('res/webroot/score_board.html'):load()
    
    local pos = ae.ARVec3:new_local(0,-2,0)

    local dir = ae.ARVec3:new_local(0,1,0)
    create_portal(pos, dir)

    for i = 1, 3, 1 do
        AR:perform_after(10000 * i, function()
            local pos = ae.ARVec3:new_local(i,-2,0)
            local dir = ae.ARVec3:new_local(0,1,0)
            create_portal(pos, dir)
        end)

    end

end

function initialize_physics()
    --初始化物理世界
    ARLOG("initialize_physics")
    gravity = ae.ARVec3:new(0,0,0)
    scene:create_physics_world(gravity)
    physics_world = scene:get_physics_world()
    --physics_world:set_debug_draw_mode(1)
end

function update_gun()
    
    local camera_matrix = scene:get_active_camera():get_property_mat44("world_transform")

    -- 计算武器的角度
    local camera_front = camera_matrix:front()

    local gun_up = ae.ARVec3:new_local(0,1,0)

    local gun_left = gun_up:cross(camera_front)

    gun_up = camera_front:cross(gun_left)

    local front = ae.ARVec4:new_local(camera_front.x, camera_front.y, camera_front.z, 0)
    local left = ae.ARVec4:new_local(gun_left.x, gun_left.y, gun_left.z, 0)
    local up = ae.ARVec4:new_local(gun_up.x, gun_up.y, gun_up.z, 0)

    -- 计算武器的位置
    local pos_offset = ae.ARVec4:new_local(-0.1, -0.15,-0.3,1)
    local gun_pos = ae.ARVec4:new_local()
    gun_pos.x = camera_matrix.w.x - 0.1
    gun_pos.y = camera_matrix.w.y - 0.15
    gun_pos.z = camera_matrix.w.z - 0.4
    gun_pos.w = 1

    local gun_mat44 = ae.ARMat44:new_local(left, up, front, gun_pos)

    --ARLOG("mat44 "..gun_mat44:to_string())

    local pos = ae.ARVec3:new_local()
    local quat = ae.ARQuat:new_local()
    local scale = ae.ARVec3:new_local()

    gun_mat44:decompose_matrix(pos, quat, scale)

    scene.sci_gun:set_property_vec3("world_position", pos)

    --因为武器的模型需要转90度才朝向前面
    local rot_offset = ae.ARQuat:new_local(ae.ARVec3:new_local(0,1,0), math.pi / 2.0)
    scene.sci_gun:set_property_quat("world_rotation_quat", quat * rot_offset)


    local mat = scene.sci_gun:get_property_mat44("world_transform")
    --ARLOG("mat44 2 "..mat:to_string())

end

-- radar demo --
radar = nil

function init_radar()
    local radar_node = scene:get_node_by_name("radar_background")
    radar_node:set_property_bool("visible", true)
    radar_node:set_update_handler(radar_update)

    local radar_point = scene:get_node_by_name("radar_point")
    local wh_ratio = screen_size.x / screen_size.y
    local height_ratio = 0.3 * wh_ratio
    radar = Radar.new(radar_point, scene, ae.ARVec2:new_local(height_ratio / 2, 0.3 / 2), ae.ARVec2:new_local(0.8, 0.9), 5)
end

function radar_update()
    radar:update_position()
end

function add_node_in_radar(node)
    if (radar ~= nil) then
        radar:add_target(node)
    end
end

function remove_node_in_radar(node)
    if (radar ~= nil) then
        radar:remove_target(node)
    end
end


function engine_update(delta)
    player_update(delta)

    portal_list_update(delta)

    monster_list_update(delta)

    bullet_shot_and_damage_check(delta)

    missile_list_update(delta)

    monster_weapon_list_update(delta)
end
